#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ast.h"
#include "parser.h"

char* concat(char *s1, char *s2)
{
  char *result = malloc(strlen(s1) + strlen(s2) + 1);
  strcpy(result, s1);
  strcat(result, s2);
  return result;
}


AST ast_node(int code, AST left, AST right)
{
  AST n = (AST) malloc(sizeof(struct tree_tag));
  if (n == NULL) {
    fprintf(stderr, "Out of memory\n");
    exit(1);
  }
  n->code = code;
  n->left = left;
  n->right = right;
  n->next = n->tru = n->fls = NULL;
  return n;
}

AST ast_let(variable var, AST expr)
{
  AST n = (AST) malloc(sizeof(struct tree_tag));
  if (n == NULL) {
    fprintf(stderr, "Out of memory\n");
    exit(1);
  }
  n->code = T_let;
  n->var = var;
  n->left = expr;
  n->next = n->tru = n->fls = NULL;
  return n;
}

AST ast_num(number num)
{
  AST n = (AST) malloc(sizeof(struct tree_tag));
  if (n == NULL) {
    fprintf(stderr, "Out of memory\n");
    exit(1);
  }
  n->code = T_const;
  n->num = num;
  n->next = n->tru = n->fls = NULL;
  return n;
}

AST ast_var(variable var)
{
  AST n = (AST) malloc(sizeof(struct tree_tag));
  if (n == NULL) {
    fprintf(stderr, "Out of memory\n");
    exit(1);
  }
  n->code = T_var;
  n->var = var;
  n->next = n->tru = n->fls = NULL;
  return n;
}

void backpatch(int n, list_tag *l) {
  char s[100];
  while (l != NULL) {
    sprintf(s, "%d", n);
    strcat(quads[l->n], s);
    l = l->next;
  }
}

int nq = 0;
int tmp = 0;

void genquads(AST ast) {

  int t, l;
  list_tag *lt;

  if (ast != NULL) {
    switch (ast->code) {
      case T_begin:
        genquads(ast->left);
        if (ast->right != NULL) {
          backpatch(nq+1, ast->left->next);
          genquads(ast->right);
          ast->next = ast->right->next;
        } else {
          ast->next = ast->left->next;
        }
        break;

      case T_print:
        genquads(ast->left);
        sprintf(quads[++nq], "par, %s, V, -", ast->left->place);
        sprintf(quads[++nq], "call, -, -, print");
        break;

      case T_let:
        genquads(ast->left);
        sprintf(quads[++nq], ":=, %s, -, %c", ast->left->place, ast->var);
        break;

     case T_const:
        sprintf(ast->place, "%d", ast->num);
        break;

     case T_var:
        sprintf(ast->place, "%c", ast->var);
        break;

      case '*':
        ++tmp;
        genquads(ast->left);
        genquads(ast->right);
        sprintf(quads[++nq], "*, %s, %s, $%d", ast->left->place, ast->right->place, tmp);
        sprintf(ast->place, "$%d", tmp);
        break;

      case '+':
        ++tmp;
        genquads(ast->left);
        genquads(ast->right);
        sprintf(quads[++nq], "+, %s, %s, $%d", ast->left->place, ast->right->place, tmp);
        sprintf(ast->place, "$%d", tmp);
        break;

      case T_for:
        t = ++tmp;
        genquads(ast->left);
        sprintf(quads[++nq], ":=, %s, -, $%d", ast->left->place, t);
        l = ++nq;
        sprintf(quads[l], ">, $%d, 0, %d", t, l + 2);
        sprintf(quads[++nq], "jump, -, -, ");
        genquads(ast->right);
        backpatch(l, ast->right->next);
        sprintf(quads[++nq], "-, $%d, 1, $%d", t, ++tmp);
        sprintf(quads[++nq], ":=, $%d, -, $%d", tmp, t);
        sprintf(quads[++nq], "jump, -, -, %d", l);
        lt = malloc(sizeof(struct list_tag));
        lt->n = l + 1;
        lt->next = NULL;
        ast->left->fls = lt;
        ast->next = ast->left->fls;
        break;
    }
  }

}

void gclose(AST ast) {
  ++nq;
  backpatch(nq, ast->next);
  backpatch(nq, ast->tru);
  backpatch(nq, ast->fls);
  sprintf(quads[nq], "endu, -, -, main");
}
